/*
 * SimpleController
 */

package demo;

import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;
import javaPlay.GameEngine;
import javaPlay.GameStateController;
import javaPlay.Keyboard;
import javaPlay.Mouse;
import javaPlay.Scene;
import javaPlay.Sprite;
import javaPlay.TileInfo;

/**
 *
 * @author VisionLab/PUC-Rio
 */
public class SimpleController implements GameStateController
{
    private Sprite playerSprite;
    private Sprite enemySprite;
    private Player player;
    private Enemy[] enemies;
    private Scene scene;    
    private final int NUM_OF_ENEMIES = 2;
    private final int TILE_WALL = 1;
    private final int TILE_EXIT = 3;
    private boolean horizontalMoveKeyPressed;
    private final int UP_FRAME = 0;
    private final int LEFT_FRAME = 2;
    private final int RIGHT_FRAME = 6;
    private final int DOWN_FRAME = 4;

    private Score score;
    
    public void load()
    {
        playerSprite = new Sprite("player.png", 8, 32, 32);
        enemySprite = new Sprite("enemy.png", 8, 32, 32);

        scene = new Scene();

        scene.loadFromFile("scene.scn");

        player = new Player();
        player.setSprite(playerSprite);

        scene.addOverlay(player);

        enemies = new Enemy[NUM_OF_ENEMIES];

        for(int i = 0 ; i < NUM_OF_ENEMIES ; i++)
        {
            enemies[i] = new Enemy();
            enemies[i].setSprite(enemySprite);
            scene.addOverlay(enemies[i]);
        }

        score = new Score();
        scene.addOverlay(score);
    }

    public void unload()
    {
 
    }

    public void start()
    {
        player.x = 92;
        player.y = 114;
        
        player.setCurrFrame(UP_FRAME);

        Point startPosEnemy1 = new Point();
        Point endPosEnemy1 = new Point();

        startPosEnemy1.x = 251;
        startPosEnemy1.y = 208;
        endPosEnemy1.x = 379;
        endPosEnemy1.y = 208;

        enemies[0].setPath(startPosEnemy1, endPosEnemy1);

        Point startPosEnemy2 = new Point();
        Point endPosEnemy2 = new Point();

        startPosEnemy2.x = 477;
        startPosEnemy2.y = 408;
        endPosEnemy2.x = 666;
        endPosEnemy2.y = 408;

        enemies[1].setPath(startPosEnemy2, endPosEnemy2);
    }

    public void step(long timeElapsed)
    {
        horizontalMoveKeyPressed = false;        

        Keyboard k = GameEngine.getInstance().getKeyboard();
        
        if(k.keyDown(Keyboard.UP_KEY) == true)
        {
            player.y--;
            player.setCurrFrame(UP_FRAME);
        }
        if(k.keyDown(Keyboard.DOWN_KEY) == true)
        {
            player.y++;
            player.setCurrFrame(DOWN_FRAME);
        }
        if(k.keyDown(Keyboard.LEFT_KEY) == true)
        {
            player.x--;
            horizontalMoveKeyPressed = true;
            player.setCurrFrame(LEFT_FRAME);
        }
        if(k.keyDown(Keyboard.RIGHT_KEY) == true)
        {
            player.x++;
            horizontalMoveKeyPressed = true;
            player.setCurrFrame(RIGHT_FRAME);
        }

        Point playerMin = new Point(player.x, player.y);
        Point playerMax = new Point(player.x + 31, player.y + 31);

        for(int i = 0 ; i < NUM_OF_ENEMIES ; i++)
        {
            enemies[i].step(timeElapsed);

            Point enemyMin = new Point(enemies[i].x, enemies[i].y);
            Point enemyMax = new Point(enemies[i].x + 31, enemies[i].y + 31);

            if(isColliding(playerMin, playerMax, enemyMin, enemyMax) == true)
            {
                // update Score
                score.setScore(-1);
                // back to start point
                player.x = 92;
                player.y = 114;
                // Shutdown if Score = 0
                if (score.getScore() == 0)
                    GameEngine.getInstance().requestShutdown();
            }
        }

        Vector tiles = scene.getTilesFromRect(playerMin, playerMax);      

        for(int i = 0 ; i < tiles.size() ; i++)
        {
            TileInfo tile = (TileInfo)tiles.elementAt(i);

            if((tile.id == TILE_WALL) && (isColliding(playerMin, playerMax,
                    tile.min, tile.max) == true))
            {
                int midx = tile.min.x/2 + tile.max.x/2;
                int midy = tile.min.y/2 + tile.max.y/2;

                if(horizontalMoveKeyPressed == true)
                {
                    if(player.x > midx)
                    {
                        player.x = tile.max.x + 1;
                    }
                    else
                    {
                        player.x = tile.min.x - 33;
                    }
                }
                else
                {
                    if(player.y > midy)
                    {
                        player.y = tile.max.y + 1;
                    }
                    else
                    {
                        player.y = tile.min.y - 33;
                    }
                }
            }
            else if(tile.id == TILE_EXIT)
            {
                GameEngine.getInstance().setNextGameStateController
                        (Global.END_SCREEN_CONTROLLER_ID);
            }
        }
    }

    public void draw(Graphics g)
    {        
        scene.draw(g);
    }

    public void stop()
    {
 
    }

    private boolean isColliding(Point min1, Point max1, Point min2, Point max2)
    {
        if(min1.x > max2.x || max1.x < min2.x)
        {
            return false;
        }
        if(min1.y > max2.y || max1.y < min2.y)
        {
            return false;
        }

        return true;
    }
}
